/*************************************************
  output.c :

          ʸϤ뤿δؿ

         1994.04.05 by Tezuka, Tadanori

*************************************************/
#include <stdio.h>
#include "toPS.h"
#include <time.h>

#define  ST_ASCII       0
#define  ST_KANJI       1
#define  ST_HANKAKU     2
#define  ST_BOLD        3
#define  ST_OBLIQUE     4
#define  ST_BOLDOBLIQUE 5
#define  ST_KEISEN      6
#define  ST_EXIT       -1

#define  PRE_ASCII       "\\377\\000"
#define  PRE_KANJI       "\\377\\001"
#define  PRE_HANKAKU     "\\377\\002"
#define  PRE_BOLD        "\\377\\003"
#define  PRE_OBLIQUE     "\\377\\004"
#define  PRE_BOLDOBLIQUE "\\377\\005"
#define  PRE_KEISEN      "\\377\\006"

static u_char *(*separate_str[])() = 
          {separate_str_euc, separate_str_sjis, separate_str_jis};

FILE   *out = stdout;
int    CommentNestLevel = 0;
int    CommentStrings   = NO;
int    Columns          = 0;

extern int Kanji_code;

static u_char *s_normal();
static void    s_ascii();

/*****************************************
  out_main()

  ʸϤᥤؿ

   arg:  buf  ... ʸ

   return value:
         0 ... ｪλ
         
*****************************************/
int out_main(buf)
u_char *buf;
{
  int status;
  int oldstatus;
  int reset = RESET;

  status    = ST_ASCII;
  oldstatus = ST_EXIT;

  /*separate_str[Kanji_code](NULL,NULL,&reset);*/
  separate_str[0](NULL,NULL,&reset);

  Columns   = 0;

  fprintf(out,"(");
  while(status != ST_EXIT){
    buf = s_normal(buf, &status,&oldstatus);
    oldstatus = status;
  }
  fprintf(out,") W CRLF\n");
  return(0);
}
      

/*****************************************
  s_normal()

   ܡʥơޥˤ
  롼סǡʸɤ߹ߤΥ
  ơѹԤʤʸ
  롣

   arg:  buf       ... ʸ
         status    ... ¹ԸΥơ
         oldstatus ... ¹Υơ

   return value:
         ʸؤΥݥ
         
*****************************************/
static u_char *s_normal(buf, status, oldstatus) 
u_char *buf;
int    *status,*oldstatus;
{
  u_char str[LBUFSIZE];
  u_char *p;
  int  s = 0;
  int  i,j;

  /* p = separate_str[Kanji_code](buf,str,&s);*/
  p = separate_str[0](buf,str,&s);

  if (s == RESET) return(p);

  if ((p == buf)&&(*p == '\0')) *status = ST_EXIT;
  else if (s == KANJI) {
    if (status != oldstatus) fprintf(out,PRE_KANJI);
    Columns += strlen(str);
    bslash_fputs(str,out);
    *status = ST_KANJI;
  } else if (s == KEISEN) {
    if (status != oldstatus) fprintf(out,PRE_KEISEN);
    Columns += strlen(str);
    bslash_fputs(str,out);
    *status = ST_KEISEN;
  } else if (s == HANKAKU) {
    if (status != oldstatus) fprintf(out,PRE_HANKAKU);
    Columns += strlen(str);
    for(i = strlen(str),j = 0;i > 0;i--,j++){
      fprintf(out,"\\%3.3o",str[j]);
    }
    *status = ST_HANKAKU;
  }
  else {
    s_ascii(str);
    *status = ST_ASCII;
  }
  return(p);
}


/*********************************************
  s_ascii()

    ST_ASCIIξˡʸ򥻥ѥ졼
  ʬ䤹

   arg:  buf  ... ʸ
         status    ... ¹ԸΥơ
         oldstatus ... ¹Υơ

   return value:
         ʤ
         
*********************************************/
static void s_ascii(buf) 
u_char *buf;
{
  extern int C_Comment;
  extern int  NoOfLine;
  u_char *p;
  u_char str[LBUFSIZE];
  u_char type;
  int    tabbing;
  int    status;
  int    LineComment = 0;

  status = ST_KANJI; 
  for(p = buf;*p != '\0';) {
    p = separate_token(p, str);   /* Token ʬ䤹 */
    if ((CommentNestLevel == 0)&&(CommentStrings == NO)&&(LineComment == 0)
	&&((type = str_reserved(str)) != 0)) {
          switch(type){
	    case BOLD         :  
	      if (status != ST_BOLD) fprintf(out,PRE_BOLD);
	      status = ST_BOLD;bslash_fputs(str,out);break;
	    case OBLIQUE      :   
	      if (status != ST_OBLIQUE) fprintf(out,PRE_OBLIQUE);
	      status = ST_OBLIQUE;bslash_fputs(str,out);break;
	    case BOLDOBLIQUE  : 
	      if (status != ST_BOLDOBLIQUE) fprintf(out,PRE_BOLDOBLIQUE);
	      status = ST_BOLDOBLIQUE;bslash_fputs(str,out);break;
	  }
    } else if (str[0] == 0x08) { /* backspace */
      if (status != ST_BOLDOBLIQUE) fprintf(out,PRE_BOLDOBLIQUE);
      status = ST_BOLDOBLIQUE;bslash_fputs(&str[1],out);
    } else {
      if (status != ST_ASCII) fprintf(out,PRE_ASCII);
      status = ST_ASCII;
      if (str[0] == '\t') { /* tabbing */
	tabbing = opt_tabwidth - Columns%opt_tabwidth;
	Columns += tabbing - 1;
	for(;tabbing != 0;tabbing--) bslash_fputs(" ",out);
      } else if ((str[0] == '/' )&&(*p == '*')&&(CommentStrings == NO)){
	p++;if (C_Comment == ON) CommentNestLevel++;
	if (CommentNestLevel >= 2){
	  fprintf(stderr,"Warn:%d:", NoOfLine);
	  fprintf(stderr,"'/*' within comment. Comment is nested!!\n");
	}
	bslash_fputs("/*",out);
      } else if ((str[0] == '*' )&&(*p == '/')) {
	p++;
#ifdef  COMMENTNEST
	if (C_Comment == ON) CommentNestLevel--;
#else
	if (C_Comment == ON) CommentNestLevel = 0;
#endif
	bslash_fputs("*/",out);
      } else if ((str[0] == '/')&&(*p == '/')&&(C_Comment == ON)) {
	LineComment = 1; p++;
	bslash_fputs("//",out);	
      } else if ((str[0] == '\"')||(str[0] == '\'')) {
	if (CommentStrings == NO) CommentStrings = str[0];
	else if (CommentStrings == str[0]) CommentStrings = NO;
	if ((C_Comment == ON)&&(CommentNestLevel != 0)) CommentStrings = NO;
	bslash_fputs(str,out);
      } else bslash_fputs(str,out);
    }
    Columns += strlen(str);
  }
  return;
}
  
  

/*****************************************
  out_header()

   إåϤ

   arg:  ʤ

   return value:
         ʤ
         
*****************************************/
void out_header()
{
  extern u_char *typename;
  FILE *fp;
  int  c;
  char buf[30],*uname;
  char *path;
  time_t clock;

  if ((path = getFilename("toPS.ps")) == NULL) {
    fprintf(stderr,"toPS : can not find file toPS.ps\n");
    exit(1);
  }

  if ((fp = fopen(path,"r")) == NULL){
    fprintf(stderr,"toPS: %s can not open!!\n", path);
    exit(1);
  }

  while((c = getc(fp)) != EOF) fputc(c,out);
  fclose(fp);

  /* # of copies */
  fprintf(out,"/#copies %d def\n", opt_NoofCopies);

  /* Time & User Name */
  if (opt_plainout == OFF) {
    time(&clock);strcpy(buf,(char *)ctime(&clock));
    buf[strlen(buf)-1] = '\0';
    fprintf(out,"%% set time and user name\n");
    fprintf(out,"/TIME (%s) def\n",buf);
    if (opt_username == ON){
      uname = (char *)getenv("USER");
      if (uname == NULL) fprintf(out,"/USER ( ) def\n");
      else fprintf(out,"/USER ( : %s) def\n",uname);
    } else fprintf(out,"/USER ( ) def\n");
    if (opt_headline == ON) fprintf(out,"/HeadLine 1 def\n");
    else fprintf(out,"/HeadLine 0 def\n");
    /* set Mode */
    fprintf(out,"/Mode (- ");bslash_fputs(typename,out);
    fprintf(out," -) def\n");
  } else {
    fprintf(out,"%% set time and user name\n");
    fprintf(out,"/TIME ( ) def\n");
    fprintf(out,"/USER ( ) def\n");
    fprintf(out,"/HeadLine 0 def\n");
    fprintf(out,"/Mode ( ) def\n");
  }

  /* set left margin */
  fprintf(out,"%d SLM", opt_LeftMargin);
  /* set multi mode  */
  if (opt_multi == ON)  fprintf(out," MM");
  /* set book mode  */
  if (opt_book  == ON)  fprintf(out," BM");
  /* blank mode */
  if (opt_blankright == ON) fprintf(out," BR");

  fprintf(out,"\n%% end \n\n");
  return;
}
